/*   
 Arduino code for simple rev. burner
 */

//    This code is emulating the signal of ignition coil.
//     ex) 4 cylinder, 4-cycle engine. (2 combustion cycle/rev.)
//
//     1 rpm = 0.5 combustion cycle/min.cylinder = 1/120 combustion cycle/sec.cylinder.
//     => 1/120 Hz/cylinder (Combustion cycle = signal for ignition So, 1 combustion cycle/sec. = 1Hz)
//     => 1 rpm with 4 cylinder = 1/120 Hz/cyclinder * 4 cylinder = 1/30 Hz
//
//     => 1 RPM = 1/120 hz/cylinder
//
//    Following the formula above, Hz vaue per 1 rpm would be..
//    * 2 cylinder = 1/60 Hz
//    * 4 cylinder = 1/30 Hz
//    * 8 cylinder = 1/15 hz/rpm

// Serial parser for Interface - Output : M<Dash2>
// Serial parser for Shutdown - Output : Z00
// M : rpm              (16bit - hexadecimal output)
// Z : clear            (to clear display and max rpm value)
//
// for dashboard
// Pin D3 : STB0 - strobe
// Pin D4 : CLK - clock
// Pin D5 : DIO - data

int i;
double cylinder = 4;           // cyliner count for Tachometer.
double divideUnit = 120;         // 1/120 hz/(rpm * cylinder)
double hzPerRpm = cylinder / divideUnit;
int rpmtone;
int pinTacho = 9;
unsigned int rpm;           // holds the rpm data (0-65535 size)
unsigned int rpmmax = 5000; // retrieves from rpm live with minimum setting on initialization and Shutdown - Output
byte rpmdata = 0;           // marker that new data are available

////////////////////////////////////////////////////////////////////////////////
// INITIALIZATION
////////////////////////////////////////////////////////////////////////////////
void setup()
{
  // serial initialization
  Serial.begin(19200);
}

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Main Loop ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void loop()
{

  // read input data
  readSerialData();

  // generate signal and send to tacho
  tacho();

}

////////////////////////////////////////////////////////////////////////////////
// Procedure: wait for complete trame
////////////////////////////////////////////////////////////////////////////////
void readSerialData()
{
  byte Data[5]={
    '0','0','0','0','0'  };

  // parse the buffer
  if (Serial.available()>4){
    Data[0]=Serial.read();                   // read command
    if (Data[0]=='M'){                       // if rpm data
      Data[1]=Serial.read();                 // store high byte
      Data[2]=Serial.read();                 //
      Data[3]=Serial.read();                 //
      Data[4]=Serial.read();                 // store low byte
      rpm = NormalizeData(Data);
      rpmdata=1;                             // we got new data!       
      if (Serial.available()>4){
        Data[0]=Serial.read();               // re-read command
      }
    }
    if (Data[0] == 'Z' ){                          // if Shutdown - Output
      Serial.flush();
      rpmmax=5000;                                 // reset rpmmax
    }             
  }
  if (Serial.available()>40) Serial.flush();
}

////////////////////////////////////////////////////////
void tacho()
////////////////////////////////////////////////////////
{   
  if (rpmdata == 1) {           
    // auto set max rpm
    // with filter of negative value
    // reported from 65535 to down in some games
    if (rpm>rpmmax && rpm<50000) {
      rpmmax=rpm;
    }
    
    // rpm gauge gets messy movements below 100 rev.
    if(rpm>50000){
      rpm = 0;
    }

    // Tacho range : from 0 to 10000, we will use 0 to 320 hz.
    //rpmtone = map(rpm,0,10000,0,320);
    rpmtone = (int)(rpm * hzPerRpm);
    
    // if hz. value goes over 320, then,,
    if(rpmtone > 320) {
      rpmtone = 333;
    }

    //It is not possible to generate tones lower than 31Hz. For technical details, 
    // see Brett Hagman's notes. (https://code.google.com/p/rogue-code/wiki/ToneLibraryDocumentation#Ugly_Details)
    if(rpmtone < 31) {
      rpmtone = 31;
    }

    // For debugging.
    if(rpm>500){    
      tone(pinTacho, rpmtone);
    }
    else{
      noTone(pinTacho);
    }
    ;

    rpmdata=0;
  }
}

////////////////////////////////////////////////////////////////////////////////
// Function: convert Hex to Dec
////////////////////////////////////////////////////////////////////////////////
unsigned int NormalizeData(byte x[5])
////////////////////////////////////////////////////////////////////////////////
{
  unsigned int result;

  for (int i=1; i<5; i++)
  {
    if (x[i]>47 && x[i]<58 ){   //for xA to xF
      x[i]=x[i]-48;
    }                       
    if (  x[i]>64 && x[i]<71 ){ //for x0 to x9
      x[i]=x[i]-55;               
    }
  }    
  result = (x[1]*16*16*16+x[2]*16*16+x[3]*16+x[4]);
  return result;
}


